/**
 * @module flitter-jobs/template/job
 */

/**
 * Get the contents of a new job file with the given class name.
 * @param {string} class_name - the name of the Job's class
 * @param {string} file_name - the canonical name
 * @returns {string}
 */
module.exports = exports = (class_name, file_name) => {
    return `const { Job } = require('flitter-jobs')

/*
 * ${class_name} Definition
 * ----------------------------------------------
 * This is an example of a job definition. The execute()
 * method is async and when its promise resolves, the job is
 * considered finished.
 *
 * Note that the job is executed from within a full instance
 * of your app, but it is NOT the same instance as the thread
 * that the main server runs on.
 *
 * You can queue these jobs by adding them to queues by name.
 * For example:
 *  const queue = this.jobs.queue('low_priority')
 *  await queue.add('${file_name}', { any: 'data here' })
 *
 * The name of the job is the unqualified canonical name of
 * the job.
 */
class ${class_name} extends Job {
    static get services() {
        return [...super.services, 'output']
    }

    /*
     * This method is called when the job is executed.
     * The first parameter is the instance of BullMQ's job class.
     */
    async execute(job) {
        // Job code here
    }
}

module.exports = exports = ${class_name}
`
}
